{% from '/schema_templates/json/content.jinja2' import content with context %}

{% block content %}
    <div class="row">
        <div class="col text-center">
            <h1>{{ title|title }}</h1>
        </div>
    </div>
    <div class="row">
        <div class="col">
            {{ content(schema) }}
        </div>
    </div>
{% endblock %}
